clear
%% LOAD DATA: AllVariables object
filename = ['/AllVariables'];
load(filename); % AllVariables

% Main vectors (from MainVariablesTreatment.m)
timeStamps                  = AllVariables.timeStamps                ;
CPI                         = AllVariables.CPI                       ;
CPI_Rate                    = AllVariables.CPI_Rate                  ;
Pop_vec                     = AllVariables.Pop_vec                   ;
LabFor                      = AllVariables.LabFor                    ;
PercLabForce                = AllVariables.PercLabForce              ;
GDP                         = AllVariables.GDP                       ;
GDP_real_pc                 = AllVariables.GDP_real_pc               ;
GDP_real_pcQrt              = AllVariables.GDP_real_pcQrt            ;  
yb_vec                      = AllVariables.yb_vec                    ;
yb_real_pc                  = AllVariables.yb_real_pc                ;
kb_vec                      = AllVariables.kb_vec                    ;
kb_real_pc                  = AllVariables.kb_real_pc                ;
kb_real_pcQrt               = AllVariables.kb_real_pcQrt             ;  
yk_real_pcQrt               = AllVariables.yk_real_pcQrt             ;  
FixPrivInv                  = AllVariables.FixPrivInv                ;
FixPrivInv_real_pc          = AllVariables.FixPrivInv_real_pc        ;   
kf_real_pcQrt               = AllVariables.kf_real_pcQrt             ;      
Cons                        = AllVariables.Cons                      ;
Cons_real_pc                = AllVariables.Cons_real_pc              ;
Cons_real_pcQrt             = AllVariables.Cons_real_pcQrt           ;  
s                           = AllVariables.s                         ;
s_real_pc                   = AllVariables.s_real_pc                 ;
s_real_pcQrt                = AllVariables.s_real_pcQrt              ;
pi                          = AllVariables.pi                        ;
pi_real_pc                  = AllVariables.pi_real_pc                ;
TotalInterestExp            = AllVariables.TotalInterestExp          ;  
TotalInterestExp_real_pc    = AllVariables.TotalInterestExp_real_pc  ;      
r                           = AllVariables.r                         ;
e                           = AllVariables.e                         ;
e_real_pc                   = AllVariables.e_real_pc                 ;
e_real_pcQrt                = AllVariables.e_real_pcQrt              ;
GovDebt                     = AllVariables.GovDebt                   ;
GovDebt_real_pc             = AllVariables.GovDebt_real_pc           ;
GovDebt_real_pcQrt          = AllVariables.GovDebt_real_pcQrt        ;  
ConsFixCap                  = AllVariables.ConsFixCap                ;
ConsFixCap_real_pc          = AllVariables.ConsFixCap_real_pc        ;  
ConsFixCap_real_pcQrt       = AllVariables.ConsFixCap_real_pcQrt     ;      
TotalAssets                 = AllVariables.TotalAssets               ;
TotalAssets_real_pc         = AllVariables.TotalAssets_real_pc       ;  
TotalAssets_real_pcQrt      = AllVariables.TotalAssets_real_pcQrt    ;

screenbar = 0.45;

% kb measure implies a bank-capital-to-bank-output ratio of roughly 12. 
% mean(kb_real_pcQrt)/mean(yb_real_pc) % 13.2050

%% LOAD DATA: GuessParams object
% From GuessParamsCalc1.m
filename = ['/GuessParams.mat'];
load(filename); % GuessParams

% Main Parameter guesses
Gamma                   = GuessParams.Gamma                     ;          
rho_f                   = GuessParams.rho_f                     ;          
sigma_f                 = GuessParams.sigma_f                   ;          
H_f                     = GuessParams.H_f                       ;      
B                       = GuessParams.B                         ;      
rho_B                   = GuessParams.rho_B                     ;          
sigma_fB                = GuessParams.sigma_fB                  ; % -0.2135 < 0.22 (paper)
alpha                   = GuessParams.alpha                     ;          
beta                    = GuessParams.beta                      ;          
v                       = GuessParams.v                         ;      
xi                      = GuessParams.xi                        ;      
r_e                     = GuessParams.r_e                       ;      
empirical_theta         = GuessParams.empirical_theta           ;                  
omega1_guess            = GuessParams.omega1_guess              ;                  
omega2_guess            = GuessParams.omega2_guess              ;                  
empirical_eta           = GuessParams.empirical_eta             ;                  
empirical_varphif_RAW   = GuessParams.empirical_varphif_RAW     ;                          
empirical_varphiB_RAW   = GuessParams.empirical_varphiB_RAW     ;                          
empirical_kappa         = GuessParams.empirical_kappa           ;                  
dep_gdp                 = GuessParams.dep_gpd                   ;          
K_Y                     = GuessParams.K_Y                       ;      
deltaT                  = GuessParams.deltaT                    ;          


LongRateChosen = 1;
if LongRateChosen == 1
    rbar                    = GuessParams.rbar                  ; % 0.0243
elseif LongRateChosen == 2
    rbar  = mean(TotalInterestExp_real_pc)/mean(s_real_pcQrt)   ; % 0.0175
elseif LongRateChosen == 3
    rbar  = mean(TotalInterestExp_real_pc) / mean(s_real_pc)    ; % 0.0044 
else
    disp("No interest type chosen")
end


% Later to be changed (according to simulated model moments)
mu_Bg                   = 0                                     ; % From paper
mu_Bb                   = -0.0014                               ; % From paper
zBg                     = mu_Bg/(1-rho_B)                       ; 
zBb                     = mu_Bb/(1-rho_B)                       ;

% Simplification
%eta                     = empirical_eta;
eta                     = 1.2;

EmpTheta                = empirical_theta;


%% Fixed objects

Ybmean          = mean(yb_real_pc)                              ; % 544.9886
Ykmean          = mean(yk_real_pcQrt)                           ; % 1.1630e+04
Y_T             = mean(GDP_real_pcQrt)                          ; % 1.2175e+04
Kbmean          = mean(kb_real_pcQrt)                           ; % 7.1966e+03
K_T             = Y_T * K_Y                                     ; % 1.0295e+05
Kkmean          = K_T - Kbmean                                  ; % 9.5755e+04
Cmean           = (1/K_Y - Gamma + 1 - deltaT) * K_T            ; % 9.1144e+03
screen          = 0.45                                          ;
s               = Kbmean*(1-xi) + B                             ; % 1.8651e+04
phi             = 0.2                                           ;

% Descriptive statistics
% Kkmean/K_T      % 0.9301 
% Kbmean/K_T      % 0.0699 
% Cmean/Y_T       % 0.7486 



%% Creation of fct deltaK(deltaB) ----------------------------------------------------

% Function of deltaK with respect to deltaB
equation_function = @(deltaK, deltaB) deltaT - deltaK * (Kkmean / K_T) - deltaB * (Kbmean / K_T);

% Initialize arrays to store results
deltaB_range = linspace(0, 0.5, 500); 
deltaK_results = zeros(size(deltaB_range));

% Iterate over deltaB values
for i = 1:length(deltaB_range)
    deltaB = deltaB_range(i);

    % Solve the equation for the current deltaB value
    options = optimset('Display', 'off');  % Turn off display
    deltaK = fsolve(@(x) equation_function(x, deltaB), 0, options);
    % Check if the solution is within the specified bounds
    if deltaK >= 0
        deltaK_results(i) = deltaK;
    end
end

% Remove zero values from deltaK_results
deltaB_range    = deltaB_range(deltaK_results > 0);
deltaK_results  = deltaK_results(deltaK_results > 0);

% Upper bounds of deltaK and deltaB
ub_deltaB = max(deltaB_range); 
ub_deltaK = max(deltaK_results); 

% Create a function of deltaK with respect to deltaB
deltaK_function = @(deltaB) interp1(deltaB_range, deltaK_results, deltaB, 'spline');



%% Creation of fct Eb(v) ----------------------------------------------------

equation_function = @(Effectb, v) -Ybmean + Effectb * (screen * exp(mu_Bg/(1-rho_B)) ...
    + (1 - screen) * exp(mu_Bb/(1-rho_B))) * (Kbmean^v);

% Initialize arrays to store results
v_range = linspace(0, 1, 100);
Effectb_results = zeros(size(v_range));

% Iterate over v values
for i = 1:length(v_range)
    v = v_range(i);
    
    % Solve the equation for the current v value
    options = optimset('Display', 'off');  % Turn off display
    Effectb = fsolve(@(x) equation_function(x, v), 0, options);
    
    % Check if the solution is within the specified bounds
    if Effectb >= 0
        Effectb_results(i) = Effectb;
    end
end

% Create a function Effectb(v) using interpolation
Effectb_function = @(v) interp1(v_range(Effectb_results > 0), ...
    Effectb_results(Effectb_results > 0), v, 'linear', 'extrap');


% Initialize arrays to store results
v_range = linspace(0, 1, 100);
Effectb_results = zeros(size(v_range));

% Iterate over v values
for i = 1:length(v_range)
    v = v_range(i);
    
    % Solve the equation for the current v value
    options = optimset('Display', 'off');  % Turn off display
    Effectb = fsolve(@(x) equation_function(x, v), 0, options);
    
    % Check if the solution is within the specified bounds
    if Effectb >= 0
        Effectb_results(i) = Effectb;
    end
end






%% SYSTEM


% (1) STANDARD ................................................................................................


% Intermediate variables
etilde_kb   = Ybmean/Kbmean -(rbar + deltaK_function(deltaB)) + (1+rbar)*xi ; 

% System
F_debug = @(deltaB,v,TR_kb,omega1,omega2,alpha,beta,theta) [...
abs(deltaT * K_T - deltaK_function(deltaB) * Kkmean - deltaB * Kbmean);                                                 % (1)
abs(-Cmean+Ybmean+Ykmean-deltaK_function(deltaB)*Kkmean-deltaB*Kbmean-(phi/2)*(screen)^2);                              % (2)
%abs(-Cmean - Gamma * (Kkmean + Kbmean) + Ybmean + Ykmean + (1--deltaK_function(deltaB))*Kkmean + (1-deltaB)*Kbmean);   % (2)
abs(-beta/Gamma+1/(alpha*(Ykmean/Kkmean)+1-deltaK_function(deltaB)));                                                   % (3)
abs(-Ybmean+Effectb_function(v)*(screen*exp(zBg)+(1-screen)*exp(zBb))*(Kbmean^v));                                      % (4)
abs(-(xi*Gamma)/(beta/Gamma)-rbar*(1-xi)+xi+v*Ybmean/Kbmean-deltaB+TR_kb*(1+omega2*(1-v)*Ybmean/Kbmean));               % (5)
abs(-phi*screen+(beta/Gamma)*Effectb_function(v)*(exp(zBg)-exp(zBb))*(Kbmean^v)*(1-omega2*TR_kb));                      % (6)
abs(-TR_kb + omega1 * exp(-omega2 * etilde_kb));                                                                        % (7)
abs((beta/Gamma)*(1+rbar)-1+(theta/Gamma)*(s^(-eta))*(Cmean^(eta-1))*(1/(1/Cmean-theta*(s^(1-eta))*(Cmean^(eta-2)))))   % (8)
    ];



% Initial Value    | Lower Bound     | Upper Bound
x0(1,1) = 0.01     ; LB(1,1) = 0.001 ; UB(1,1) = 0.045 ; % deltaB
x0(2,1) = 0.45     ; LB(2,1) = 0.1   ; UB(2,1) = 0.6   ; % v
x0(3,1) = 0.1      ; LB(3,1) = 0     ; UB(3,1) = 1     ; % TR_kb
x0(4,1) = 0.5      ; LB(4,1) = 0.01  ; UB(4,1) = 5     ; % omega1
x0(5,1) = 2        ; LB(5,1) = 10    ; UB(5,1) = 50    ; % omega2
x0(6,1) = alpha    ; LB(6,1) = 0.2   ; UB(6,1) = 0.4   ; % alpha
x0(7,1) = beta     ; LB(7,1) = 0.9   ; UB(7,1) = 1     ; % beta
x0(8,1) = EmpTheta ; LB(8,1) = 0.01  ; UB(8,1) = 1     ; % theta

% Solver
x = lsqnonlin(@(x) F_debug(x(1),x(2),x(3),x(4),x(5),x(6),x(7),x(8)),x0,LB,UB);
param_names = {'deltaB', 'v', 'TR_kb', 'omega1', 'omega2', 'alpha', 'beta', 'theta'};
F_debug_result = F_debug(x(1), x(2), x(3), x(4), x(5), x(6), x(7), x(8));
ParamSyst = x;

output_table = table(ParamSyst, LB, UB, F_debug_result, ...
    'VariableNames',{'Value', 'LowerBound', 'UpperBound', 'Residual'}, ...
    'RowNames',param_names');
disp(output_table);


deltaB   = ParamSyst(1)                                         ; % 0.1093     
v        = ParamSyst(2)                                         ; % 0.4747 
TR_kb    = ParamSyst(3)                                         ; % 0.0543     
omega1   = ParamSyst(4)                                         ; % 0.9908     
omega2   = ParamSyst(5)                                         ; % 18.1462
alpha    = ParamSyst(6)                                         ; % 0.3995
beta     = ParamSyst(7)                                         ; % 0.9383
theta    = ParamSyst(8)                                         ; % 0.0252
deltaK   = deltaK_function(ParamSyst(1))                        ; % 0.0138
Effectb  = Effectb_function(ParamSyst(2))                       ; % 8.0918

deltaK_str = num2str(deltaK);
Effectb_str = num2str(Effectb);
disp(['deltaK  : ', deltaK_str]);
disp(['Effectb : ', Effectb_str]);
disp("---------------------------------------------------------------")
disp("---------------------------------------------------------------")


varphif   = empirical_varphif_RAW                               ; % (16)
varphiB   = empirical_varphiB_RAW                               ; % (17)
kappa     = empirical_kappa                                     ; % (18)
Effectk   = ((Ykmean/(Kkmean^alpha))^(1/(1-alpha)))/(H_f)       ; % (25) 1569.8 > 17...
mu_K      = 0                                                   ; % (26)        
sigmaBg   = 0.0799                                              ; % (27) 
sigmaBb   = 0.0808                                              ; % (28)




%% Steady states


screen_S  = 0.45                                                    ; % (1)  0.4500
zBg_S     = mu_Bg/(1-rho_B)                                         ; % (2)  0
zBb_S     = mu_Bb/(1-rho_B)                                         ; % (3)  -0.0118
zK_S      = 1                                                       ; % (4)  1
Lambda_S  = 1                                                       ; % (5)  1
M_S       = Gamma^(-1) * beta                                       ; % (6)  0.9533
yk_S      = Ykmean                                                  ; % (7)  1.1630e+04
yb_S      = Ybmean                                                  ; % (8)  544.9886
kk_S      = Kkmean                                                  ; % (9)  9.5755e+04
kb_S      = Kbmean                                                  ; % (10) 7.1966e+03  
Y_S       = yb_S + yk_S                                             ; % (11) 1.2175e+04
e_S       = Kbmean * xi                                             ; % (12) 610.3237  
s_S       = B + Kbmean - e_S                                        ; % (13) 1.8651e+04  
A_S       = s_S + e_S                                               ; % (14) 1.9262e+04  
K_S       = kk_S + kb_S                                             ; % (15) 1.0295e+05  
rk_S      = alpha * (yk_S / kk_S)                                   ; % (16) 0.0537 
rgov_S    = rbar                                                    ; % (17) 0.0243 
r_S       = rbar                                                    ; % (18) 0.0243 
rb_S      = v * yb_S / kb_S                                         ; % (19) 0.0352 
w_k_S     = (1 - alpha) * (yk_S / H_f)                              ; % (20) 0.0352  
c_S       = (1/(K_Y) - Gamma + 1 - deltaT) * K_T                    ; % (21) 9.1144e+03  
pi_S      = yb_S - deltaB * kb_S + rgov_S * B - r_S* s_S            ; % (22) 13.9535
etilde_S  = pi_S + e_S                                              ; % (23) 624.2772
TR_S      = omega1 * kb_S * exp(-omega2 + etilde_S/kb_S )           ; % (24) 3.8510
T_S       = TR_S + (1 + rgov_S) * B - Gamma * B                     ; % (25) 185.5330
d_S       = etilde_S + TR_S - Gamma*e_S - (phi/2)*screen_S^2        ; % (26) 12.1494
p_S       = d_S * (M_S * Gamma / (1 - M_S * Gamma))                 ; % (27) 308.6842
mu_S      = Gamma - M_S * (1 + r_S) * (1 - omega1 * TR_S / kb_S)    ; % (28) 0.0330
n1        = c_S + Gamma*s_S + Gamma * kk_S + p_S                    ; 
n2        = d_S + p_S + (1 + r_S) * s_S + (w_k_S * H_f) ...     
             + (rk_S + 1 - deltaK) * kk_S - T_S                     ;
n         = min([n1, n2])                                           ; 
n_S       = n                                                       ; % (29) 1.2489e+05
inv_k_S   = kk_S-(1-deltaK)*kk_S/Gamma                              ; % (30) 2.5995e+03
inv_b_S   = kb_S-(1-deltaB)*kb_S/Gamma                              ; % (31) 433.4885


% InitSteadyState = [...
% Y_S      ;
% yk_S     ;
% yb_S     ;
% A_S      ;
% s_S      ;
% T_S      ;
% e_S      ;
% c_S      ;
% d_S      ;
% kk_S     ;
% kb_S     ;
% K_S      ;
% TR_S     ;
% M_S      ;
% n_S      ;
% etilde_S ;
% p_S      ;
% r_S      ;
% rk_S     ;
% rb_S     ;
% w_k_S    ;
% rgov_S   ;
% mu_S     ;
% screen_S ;
% pi_S     ;
% zBg_S    ;
% zBb_S    ;
% zK_S     ;
% Lambda_S ;
% inv_k_S  ;
% inv_b_S  %;
% %Welfare_S
% ];








%% DYNARE

dbar      = d_S                                                     ; % (29th) params

Guess       = [Y_S,         yk_S,   yb_S,   A_S,        s_S,    ... (5)   
               T_S,         e_S,    c_S,    d_S,        kk_S,   ... (10)
               kb_S,        K_S,    TR_S,   M_S,        n_S,    ... (15)
               etilde_S,    p_S,    r_S,    rk_S,       rb_S,   ... (20)
               w_k_S,       rgov_S, mu_S,   screen_S,   pi_S,   ... (25)
               zBg_S,       zBb_S,  zK_S,   Lambda_S,   inv_k_S,... (30)
               inv_b_S] ; %                                         (31) 



% Current nomenclature, not from .mod file 
parameter   = [Gamma    ,rho_f    ,sigma_f  ,H_f      ,B        ,...(5 ) 
               rho_B    ,sigma_fB ,alpha    ,beta     ,v        ,...(10) 
               xi       ,theta    ,omega1   ,omega2   ,eta      ,...(15) 
               varphif  ,varphiB  ,kappa    ,mu_Bg    ,mu_Bb    ,...(20) 
               phi      ,deltaB   ,deltaK   ,Effectb  ,Effectk  ,...(25) 
               mu_K     ,sigmaBg  ,sigmaBb  ,dbar                ...(29)
               ];

save param_xx Gamma         rho_f           sigma_f     H_f         B           ...
              rho_B         sigma_fB        alpha       beta        v           ...
              xi            theta           omega1      omega2      eta         ...
              varphif       varphiB         kappa       mu_Bg       mu_Bb       ...
              phi           deltaB          deltaK      Effectb     Effectk     ...
              mu_K          sigmaBg         sigmaBb     dbar                    ...
              Y_S           yk_S            yb_S        A_S         s_S         ...
              T_S           e_S             c_S         d_S         kk_S        ...
              kb_S          K_S             TR_S        M_S         n_S         ...
              etilde_S      p_S             r_S         rk_S        rb_S        ...
              w_k_S         rgov_S          mu_S        screen_S    pi_S        ...
              zBg_S         zBb_S           zK_S        Lambda_S    inv_k_S     ...
              inv_b_S;


dynare RBC_model.mod noclearall nolog nograph


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEST loop
% Loop over values of eta and see effects on consumption c (stored in
% c_storage)

% Read out variable position 
c_pos           = strmatch('c',M_.endo_names,'exact');
eta_range       = 1:0.02:1.5;
c_storage       = zeros(length(eta_range), 1);
param_change    = zeros(length(eta_range), 1); % for testing

for ii = 1:length(eta_range)
    %set_param_value('eta',eta_range(ii))
    M_.params(strmatch('eta', M_.param_names,'exact')) = eta_range(ii) ;

    % Set seed
    set_dynare_seed('default');  % (If you have this option in your code you 
    % need not use the options_.noprint=1 statement above)
    try 
        %[info, oo_] = stoch_simul(M_, options_, oo_, var_list_);
        [steady_state,params,info] = evaluate_steady_state(oo_.steady_state,M_,options_,oo_,~options_.steadystate.nocheck);
        param_change(ii) = params(strmatch('eta',M_.param_names,'exact')); % for testing
    catch 
        warning('Assigning a value of NaN to info.');
        info = NaN; 
    end  
    if info ==0
        c_storage(i,1) = steady_state(c_pos);
    else 
        fprintf('Here there is an error with this combination of paramters!\n');
        c_storage(i,1) = NaN;
         %variance.pi(ii,ii)=NaN;
    end
end











