close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     VARIABLES AND PARAMETERS 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% In order
var %----------------------------------------------------------------------
Y           (long_name='Aggregate output'),                              % (1 )  
yk          (long_name='Output of Firms'),                               % (2 )  
yb          (long_name='Output of Banks'), % yb instead of yB.           % (3 )  
A           (long_name='Aggregate Bank Assets'),                         % (4 )  
s           (long_name='Saving deposits'),                               % (5 )  
T           (long_name='Taxes'),                                         % (6 )  
e           (long_name='Equity'),                                        % (7 )  
c           (long_name='Consumption'),                                   % (8 )  
d           (long_name='Dividends'),                                     % (9 )  
kk          (long_name='Capital of Firms'),                              % (10)  
kb          (long_name='Bank Capital'),                                  % (11)  
K           (long_name='Aggregate Capital'),                             % (12)  
TR          (long_name='Transfer function'),                             % (13)  
M           (long_name='Stochastic Discount Factor'),                    % (14)  
n           (long_name='Net Worth'),                                     % (15)  
etilde      (long_name='Equity After Profits'),                          % (16)  
p           (long_name='Price of stock'),                                % (17)  
r           (long_name='Interest rate of savings deposit'),              % (18)  
rk          (long_name='Rental rate of capital'),                        % (19)  
rb          (long_name='Interest rate of Banks'),                        % (20)  
w_k         (long_name='Wage rate'),                                     % (21)  
rgov        (long_name='Government interest rate'),                      % (22)  
mu          (long_name='Kuhn Tucker condition'),                         % (23)  
screen      (long_name='Monitoring intensity'),                          % (24)  
pi          (long_name='Profit of Banks'),                               % (25)  
zBg         (long_name='Good Bank productivity'),                        % (26)  
zBb         (long_name='Bad Bank productivity'),                         % (27)  
zK          (long_name='Firm productivity'),                             % (28)  
Lambda      (long_name='Lagrangian Multiplier'),                         % (29)  
inv_k       (long_name='Investment of Firms'),                           % (30)  
inv_b       (long_name='Investment of Banks'), % inv_b instead of inv_B   % (31)  

% Info variables
yb_Y ,
kb_yb,
C_RC 

; 

varexo %-------------------------------------------------------------------
eB, eK
;



parameters %---------------------------------------------------------------
Gamma       ,
rhoK        ,
sigmaK      ,
Nk_N        ,
B           ,
rhoB        ,
rhoBK       ,
alpha       ,
beta        ,
v           ,
xi          ,
theta       ,
omega1      ,
omega2      ,
eta         ,
varphik     ,
varphib     ,
kappa       ,
mu_Bg       ,
mu_Bb       ,
phi         ,
deltaB      ,
deltaK      ,
Effectb     ,
Effectk     ,
mu_K        ,
sigmaBg     ,
sigmaBb     ,
dbar        
; % 29





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                            CALIBRATION 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load param_xx;


                % Curr.                   % MainModel1.m
set_param_value('Gamma'   ,Gamma    ) ; % Gamma         
set_param_value('rhoK'    ,rho_f    ) ; % rho_f         
set_param_value('sigmaK'  ,sigma_f  ) ; % sigma_f       
set_param_value('Nk_N'    ,H_f      ) ; % H_f           
set_param_value('B'       ,B        ) ; % B             
set_param_value('rhoB'    ,rho_B    ) ; % rho_B         
set_param_value('rhoBK'   ,sigma_fB ) ; % sigma_fB      
set_param_value('alpha'   ,alpha    ) ; % alpha         
set_param_value('beta'    ,beta     ) ; % beta          
set_param_value('v'       ,v        ) ; % v             
set_param_value('xi'      ,xi       ) ; % xi            
set_param_value('theta'   ,theta    ) ; % theta         
set_param_value('omega1'  ,omega1   ) ; % omega1        
set_param_value('omega2'  ,omega2   ) ; % omega2        
set_param_value('eta'     ,eta      ) ; % eta           
set_param_value('varphik' ,varphif  ) ; % varphif       
set_param_value('varphib' ,varphiB  ) ; % varphiB       
set_param_value('kappa'   ,kappa    ) ; % kappa         
set_param_value('mu_Bg'   ,mu_Bg    ) ; % mu_Bg         
set_param_value('mu_Bb'   ,mu_Bb    ) ; % mu_Bb         
set_param_value('phi'     ,phi      ) ; % phi           
set_param_value('deltaB'  ,deltaB   ) ; % deltaB        
set_param_value('deltaK'  ,deltaK   ) ; % deltaK        
set_param_value('Effectb' ,Effectb  ) ; % Effectb       
set_param_value('Effectk' ,Effectk  ) ; % Effectk       
set_param_value('mu_K'    ,mu_K     ) ; % mu_K          
set_param_value('sigmaBg' ,sigmaBg  ) ; % sigmaBg       
set_param_value('sigmaBb' ,sigmaBb  ) ; % sigmaBb       
set_param_value('dbar'    ,dbar     ) ; % dbar       



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                               MODEL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Source: THESIS MASTER/Documents to Monnet/Dynare 1/rev_02132018.mod 


model;

# deltaB_Gadj = 1-(1-deltaB)/Gamma;  % Paper: deltaB_Gadj = Gamma - 1 + delta 
# deltaK_Gadj = 1-(1-deltaK)/Gamma; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bank independent sector (5 equations)

[name='1. Law of motion of capital']
%inv_k      = Gamma*kb - (1-deltaK)*kk(-1);
inv_k       = kk-(1-deltaK)*kk(-1)/Gamma ; % FOLLOWING CODE, NOT THE PHD and paper.

[name='2. Production function']
%yk         = zK*(kk(-1)^alpha)*((Effectk*Nk_N))^(1-alpha) ;
yk          = exp(zK)*(kk(-1)^alpha)*((Effectk*Nk_N))^(1-alpha);


[name='3. Productivity level']
%log(zK)    = mu_K + rhoK * log(zK(-1)) + sigmaK*eK ;
zK          = mu_K + rhoK * zK(-1) + sigmaK*eK ;

[name='4. Rental rate of capital']
%rk         = alpha* yk/kk(-1) ;
rk          = alpha* yk/kk(-1) ; 

[name='5. Wage rate']
%w_k        = (1-alpha) * yk/Nk_N ;
w_k         = (1-alpha) * yk/Nk_N ;

% Note that Nk_N = H_f


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bank dependent sector (15 equations)

[name='6. Law of motion of capital']
%inv_B     = Gamma*kb-(1-deltaB)*kb(-1) ; 
inv_b       = kb-(1-deltaB)*kb(-1)/Gamma ; 

[name='7. Profits']
pi          = yb - deltaB* kb(-1) + (rgov(-1))* (B) -  (r(-1))* (s(-1)) ; 

[name='8. Bank output']
yb          = Effectb * (screen(-1)*exp(zBg)+(1-screen(-1))*exp(zBb)) * (kb(-1)^v) ;


[name='9. Dividends']
%d          = etilde + TR - Gamma*e   
%               - (kappa/2) * (d - dbar)^2 
%               - varphib * Gamma * kb * (inv_B/kb - deltaB_Gadj)^2
%               - (phi/2) * (screen)^2 ; 

% d         = etilde + TR - Gamma*e   
%                 - (kappa/2) * (d - dbar)^2 
%                 - varphib * Gamma * kb * (inv_b/kb - deltaB_Gadj)^2    
%                 - (phi1/2)*(screen-screenbar)^2  - phi2*screen  ; 

% d           = etilde - (kappa/2) * (d - dbar)^2 + TR 
%                 - varphib * (inv_b/kb - deltaB_Gadj)^2 * Gamma*kb
%                 - Gamma*e - (phi/2)*screen^2                    ;

% NOTE chagne in time subscript. Only then, Gamma makes sense. It also works.
d           = etilde - (kappa/2) * (d - dbar)^2 + TR 
                - varphib * (inv_b(+1)/kb(+1) - deltaB_Gadj)^2 * Gamma*kb(+1)
                - Gamma*e(+1) - (phi/2)*screen(+1)^2                    ;
   

            
[name='10. Balance sheet']
e + s       = kb + B    ; 

[name='11. Capital requirements']
e           = xi * kb   ;

[name='12. Transfer function']
TR         = omega1 * kb(-1) * exp( -omega2 * (etilde/kb(-1)) ) ; 

[name='13. Equity after profits']
etilde      = e(-1) +  pi ;

[name='14. Good productivity level']
%log(zBg)   = mu_Bg +  rhoB * log(zBg(-1)) + sigmaBg*eB ; % appendix
zBg         = mu_Bg +  rhoB * zBg(-1) + sigmaBg*eB;

[name='15. Bad productivity level']
%log(zBb)   = mu_Bb +  rhoB * log(zBb(-1)) + sigmaBb*eB ; % appendix
zBb         = mu_Bb +  rhoB * zBb(-1) + sigmaBb*eB;

[name='16. FOC derivation of d']
1           = Lambda*(1+kappa*( d - dbar )); 

[name='17. FOC derivation of m^B']
% Lambda*phi*screen =  (M*Lambda(+1))*Effectb*(zBg(+1)-zBb(+1))*(kb^v) *(1-omega2*TR(+1)/kb) ;

%Lambda*(phi1*(screen-screenbar) + phi2) =  (M*Lambda(+1))*Effectb*(exp(zBg)-exp(zBb))*(kb^v)
%                         *(1-omega1*TR(+1)/kb)  ;

Lambda*phi*screen = (M*Lambda(+1))*Effectb*(exp(zBg)-exp(zBb))*(kb^v) *(1-omega1*TR(+1)/kb) ; 


[name='18. FOC derivation of b and s']
0           = M*Lambda(+1)*(rgov - r);

[name='19. FOC derivation of kb']
%mu*xi      = -Lambda* ( varphib*(inv_B/kb - deltaB_Gadj)^2
%               + 2*varphib*(inv_B/kb- (1-deltaB_Gadj))*Gamma*(1-deltaB)*(kb(-1)/(kb)) )
%               + (M*Lambda(+1)) * ( (v*(yB(+1)/kb)-r-deltaB) * (1-omega2*(TR/kb)) 
%               + 2*Gamma*varphib*(inv_B(+1)/kb(+1) - deltaB_Gadj)*(1-deltaB)
%               + TR/kb *(1+omega2*(etilde(+1)/kb)) ) ;

% mu*xi + Lambda*varphib*((inv_b/kb-deltaB_Gadj)^2 +
%               2*(inv_b/kb-deltaB_Gadj)*(1-deltaB)*(kb(-1)/(kb)))
%             = (M*Lambda(+1))*( (rb(+1)- r-deltaB) 
%                 + 2*varphib*(inv_b(+1)/kb(+1) - deltaB_Gadj)*(1-deltaB)
%                 + (TR(+1)/kb)*(1+omega1*((1-v)*yb(+1)/kb +(1+r)*e/kb )))  ;

mu*xi = -Lambda * (varphib * Gamma * (inv_b/kb - deltaB_Gadj)^2 
    + 2 * varphib * (inv_b/kb - deltaB_Gadj)^2 * Gamma * (1-deltaB) * kb(-1)/kb ) 
        + ( M + Lambda(+1)*( (v * (yb(+1)/kb) - r - deltaB) * (1-omega2*(TR(+1)/kb)) ) 
            + 2*Gamma*varphib*(inv_b(+1)/kb(+1) - deltaB_Gadj)*(1-deltaB) 
            + (TR(+1)/kb) * (1 + omega2*etilde(+1)/kb ) ) ;

% We use TR(+1)/kb instead of TR/kb. TR(t) depends on e(t+1) or e(t)


[name='20. FOC derivation of e']
%mu         = Lambda*Gamma-(M*Lambda(+1))*(1+r)*(1-omega2*TR/kb) ; % From paper.
mu          = Lambda*Gamma-(M*Lambda(+1))*(1+r)*(1-omega1*TR(+1)/kb);  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Households (7 equations)

[name='21. Stochastic discount factor']
%M          = beta*(Gamma^-1)*((1/c(+1) - theta*(s(+1)^(1-eta) )*(c(+1))^(eta-2))/
%                    (1/c - theta*(s^(1-eta))*(c^(eta-2)))) ;
M           = beta*(Gamma^-1)*((1/c(+1) - theta*(s(+1)^(1-eta) )*(c(+1))^(eta-2))/
                     (1/c - theta*(s^(1-eta))*(c^(eta-2)))) ;

[name='22. Net worth']
%n          = d+p + (1+(r(-1)))*(s(-1))+ (w_k*(Nk_N)) +(rk+1-deltaK)*kk(-1)-T ; 
n           = d+p + (1+(r(-1)))*(s(-1))+ (w_k*(Nk_N)) +(rk+1-deltaK)*kk(-1)-T ; 

[name='23. Budget constraint']
%n          = c + Gamma*s +Gamma*(1+varphik*((inv_k/kk-(1-(1-deltaK)/Gamma))^2))*kk + p    ; 
n           = c + Gamma*s +Gamma*(1+varphik*((inv_k/kk-deltaK_Gadj)^2))*kk + p    ; 

[name='24. FOC derivation of s']
%1          = M*(1+r)+ (theta/Gamma)*(s^(-eta))*(c^(eta-1)) *
%                   (1/(1/c - theta*(s^(1-eta))*(c^(eta-2)))); 
1           = M*(1+r)+ (theta/Gamma)*(s^(-eta))*(c^(eta-1))*
                    (1/(1/c - theta*(s^(1-eta))*(c^(eta-2)))); 

[name='25. FOC derivation of kk']
1+varphik*Gamma*(inv_k/kk- deltaK_Gadj)^(2) + 2*varphik*Gamma*(inv_k/kk- deltaK_Gadj)*(1-deltaK)*kk(-1)/kk
           = (M)*(rk(+1)+(1+2*varphik*Gamma*(inv_k(+1)/kk(+1) - deltaK_Gadj))*(1-deltaK)) ;

% 1+varphik*((inv_k/kk-deltaK_Gadj)^2) + 2*varphik*(inv_k/kk-deltaK_Gadj)*(1-deltaK)*(kk(-1)/(kk*Gamma))
%             = (M)*(rk(+1)+(1+2*varphik*(inv_k(+1)/kk(+1) - deltaK_Gadj))*(1-deltaK));

% We use the definition of the paper

[name='26. FOC derivation of household stock share of banks']
1           = M*Gamma*(d(+1)+p(+1))/ p ; % as in code

[name='27. Rental rate of bank dependent capital']
rb          = v* yb/kb(-1) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Government (1 equation)

[name='28. Government budget']
TR +  B*(1+rgov(-1))  = Gamma*B+ T ;

[name='29. Aggregate output']
Y = yk + yb                ; 


[name='30. Total assets']
A = B +  kb                ; 

[name='31. Aggregate Capital']
K = kk + kb ;

%[name='32. Welfare']
%Welfare  = log(c) + theta * ((s/c)^(1-eta))/(1-eta) + beta*Welfare(+1) ; 


yb_Y    = yb/Y;
kb_yb   = kb/yb;
C_RC    = Y + (1-deltaK) * kk + (1-deltaB) * kb - Gamma * K - phi/2*(screen)^2; 



end; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           STEADY STATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



initval;


Y       = Y_S       ;
yk      = yk_S      ;
yb      = yb_S      ;
A       = A_S       ;
s       = s_S       ;
T       = T_S       ;
e       = e_S       ;
c       = c_S       ;
d       = d_S       ;
kk      = kk_S      ;
kb      = kb_S      ;
K       = K_S       ;
TR      = TR_S      ;
M       = M_S       ;
n       = n_S       ;
etilde  = etilde_S  ;
p       = p_S       ;
r       = r_S       ;
rk      = rk_S      ;
rb      = rb_S      ;
w_k     = w_k_S     ;
rgov    = rgov_S    ;
mu      = mu_S      ;
screen  = screen_S  ;
pi      = pi_S      ;
zBg     = zBg_S     ;
zBb     = zBb_S     ;
zK      = zK_S      ;
Lambda  = Lambda_S  ;
inv_k   = inv_k_S   ;
inv_b   = inv_b_S   ;

eK      =   0       ;
eB      =   0       ;


% Information variables
yb_Y    = yb/Y;
kb_yb   = kb/yb;
C_RC    = Y + (1-deltaK) * kk + (1-deltaB) * kb - Gamma * K - phi/2*(screen)^2; 

end;



steady(solve_algo=0); 

shocks;
var eB; stderr 1;
var eK; stderr 1;
var eB,eK =  rhoBK;   % SigmafB
end;


options_.nocorr         =1; % Correlation table not displayed
options_.noprint        =1; % Don’t print anything. Useful for loops.
options_.nofunctions    =1; % Don't print the coefficients of the approximated solution (printing them is the default)



stoch_simul(irf = 40, order = 1, periods = 1000, drop =300,  simul_replic=100, pruning); % Works






